@@ -3,14 +3,15 @@ package Lemonldap::NG::Portal::Auth::SAML;
 use strict;
 use MIME::Base64 qw/encode_base64/;
 use Mouse;
+use HTML::Entities qw(encode_entities);
 use Lemonldap::NG::Portal::Lib::SAML;
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
   PE_CONFIRM
   PE_IDPCHOICE
   PE_LOGOUT_OK
   PE_REDIRECT
-  PE_OK
   PE_SAML_ART_ERROR
   PE_SAML_CONDITIONS_ERROR
   PE_SAML_DESTINATION_ERROR
@@ -24,7 +25,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.14';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Auth
@@ -297,9 +298,13 @@ sub extractFormInfo {
             }
 
             # Get NameID
-            my $nameid         = $login->nameIdentifier;
-            my $nameid_content = $nameid->content;
+            my $nameid = $login->nameIdentifier;
+            unless ($nameid) {
+                $self->userLogger->error("No NameID element found");
+                return PE_SAML_SSO_ERROR;
+            }
 
+            my $nameid_content = $nameid->content;
             unless ($nameid_content) {
                 $self->userLogger->error("No NameID value found");
                 return PE_SAML_SSO_ERROR;
@@ -338,9 +343,14 @@ sub extractFormInfo {
                         $self->logger->debug(
                             "Found value $value for attribute $userAttribute");
                     }
+                    else {
+                        $self->logger->warn(
+"No value for $userAttribute found in SAML assertion"
+                        );
+                    }
                 }
                 else {
-                    $self->logger->debug(
+                    $self->logger->warn(
                         "No attributes found in SAML assertion");
                 }
             }
@@ -763,8 +773,11 @@ sub extractFormInfo {
                 $req->postFields( { 'SAMLResponse' => $slo_body } );
 
                 # RelayState
-                $req->postFields->{'RelayState'} = $relaystate
-                  if ($relaystate);
+                if ($relaystate) {
+                    $req->{postFields}->{'RelayState'} =
+                      encode_entities($relaystate);
+                    $req->data->{safeHiddenFormValues}->{RelayState} = 1;
+                }
 
                 # TODO: verify this
                 push @{ $req->steps }, 'autoPost';
@@ -1123,8 +1136,11 @@ sub extractFormInfo {
         }
 
         # RelayState
-        $req->{postFields}->{'RelayState'} = $login->msg_relayState
-          if ( $login->msg_relayState );
+        if ( $login->msg_relayState ) {
+            $req->{postFields}->{'RelayState'} =
+              encode_entities( $login->msg_relayState );
+            $req->data->{safeHiddenFormValues}->{RelayState} = 1;
+        }
 
         # TODO: verify this
         $req->steps( ['autoPost'] );
@@ -1137,7 +1153,7 @@ sub extractFormInfo {
 }
 
 sub authenticate {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setAuthSessionInfo {
@@ -1252,7 +1268,7 @@ sub setAuthSessionInfo {
     $req->data->{_lassoLogin} = $login;
     push @{ $req->steps }, sub { $self->authFinish(@_) };
 
-    PE_OK;
+    return PE_OK;
 }
 
 # Inserted in $req->steps by authenticate()
@@ -1385,8 +1401,11 @@ sub authLogout {
         $req->postFields( { 'SAMLRequest' => $slo_body } );
 
         # RelayState
-        $req->postFields->{'RelayState'} = $logout->msg_relayState
-          if ( $logout->msg_relayState );
+        if ( $logout->msg_relayState ) {
+            $req->{postFields}->{'RelayState'} =
+              encode_entities( $logout->msg_relayState );
+            $req->data->{safeHiddenFormValues}->{RelayState} = 1;
+        }
 
         # Post done in Portal
         $req->steps( [ 'deleteSession', 'autoPost' ] );
@@ -1410,6 +1429,15 @@ sub authLogout {
         # Create Logout object
         $logout = $self->createLogout( $self->lassoServer );
 
+        # Do we check signature?
+        my $checkSLOMessageSignature =
+          $self->conf->{samlIDPMetaDataOptions}->{$idpConfKey}
+          ->{samlIDPMetaDataOptionsCheckSLOMessageSignature};
+
+        unless ($checkSLOMessageSignature) {
+            $self->disableSignatureVerification($logout);
+        }
+
         # Process logout response
         my $result = $self->processLogoutResponseMsg( $logout, $response );
 
@@ -1443,7 +1471,7 @@ sub handleAuthRequests {
     if ( $req->uri =~ $self->sloRe ) {
         return $self->extractFormInfo($req);
     }
-    PE_OK;
+    return PE_OK;
 }
 
 # TODO: authForce
